/*
 * Communication_CDC.c
 *
 *  Created on: 2025/01/09
 *      Author: Revive
 */

#include "Communication_CDC.h"

#include "usb_device.h"

#include "Execute_Command_LED.h"

#include <string.h>


//------------------------------------------------------
// Communication_CDC用パラメータの構造体の宣言
//------------------------------------------------------
struct sParam_Communication_CDC
                    GLB_structParam_Communication_CDC;


// CDC通信処理用 送信データ
uint8_t GLB_uint8_aCDC_SendData[APP_TX_DATA_SIZE];

// CDC通信処理用 受信データ
uint8_t GLB_uint8_aCDC_ReceiveData[APP_RX_DATA_SIZE];



//----------------------------------------------------------
// Communication_CDC用パラメータの構造体の初期化
//----------------------------------------------------------
// 引数：
// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//    なし
//----------------------------------------------------------
void Initialize_sParam_Communication_CDC(
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC)
{
  pstructParam_Communication_CDC->
        int16_vTimer_CDC_Rcv_End = -1;  // CDC受信終了タイマ
  pstructParam_Communication_CDC->
        uint16_vLength_CDC_Rcv = 0;     // CDC受信データ数

  pstructParam_Communication_CDC->
        int16_vSemaphore_CDC_Send = 0;  // CDC送信セマフォ

  pstructParam_Communication_CDC->
        uint16_vCDC_SendLength;     // CDC送信データ数
  pstructParam_Communication_CDC->
        uint16_vCDC_RcvLength = 0;  // CDC受信データ数


  //--------------------------------------------------------
  // CDC初期化フラグ
  //--------------------------------------------------------
  //  0以下: 初期化されていない。
  //  1以上: 初期化済み
  //--------------------------------------------------------
  pstructParam_Communication_CDC->int16_vCDC_Initialize = -1;
                  // 初期値: "初期化されていない。" をセット
  //--------------------------------------------------------
}


//----------------------------------------------------------
// CDC通信パラメータ初期化 と CDCの初期化
//----------------------------------------------------------
// 引数：
// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//    なし
//----------------------------------------------------------
void Initialize_Communication_CDC(
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC)
{
  //------------------------------------------------------
  // Communication_CDC用パラメータの構造体の初期化
  //------------------------------------------------------
  // 引数：
  // struct sParam_Communication_CDC
  //                  *pstructParam_Communication_CDC :
  //          Communication_CDC用パラメータの構造体のポインタ


  // 戻り値：
  //    なし
  //--------------------------------------------------------
  Initialize_sParam_Communication_CDC(
                            pstructParam_Communication_CDC);


  //------------------------------------
  // CDC初期化 : USB Device初期化
  //------------------------------------
  MX_USB_DEVICE_Init();
}



//----------------------------------------------------------
// CDC通信処理
//----------------------------------------------------------
// 処理内容 :

//  1. CDCの初期化がされていない場合、初期化します。
//  2. 受信が発生するのを待ち、受信データを取得します。
//  3. 受信データを判定して、内容に従い処理を行います。
//  4. 受信データによる処理の結果、
//     送信データがセットされた場合、送信処理を行います。
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Communication_CDC(uint8_t *puint8_JobNum,
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC)
{
  static uint8_t STC_uint8_JobNum_CDC;

  int16_t int16_ReceiveLength;

  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    //--------------------------------------
    // 初期化フラグの判定
    //--------------------------------------
    if(pstructParam_Communication_CDC->int16_vCDC_Initialize <= 0){
      //----------------------------------------------------
      // 初期化済みでない場合、
      // CDC通信パラメータ初期化 と CDC初期化を行います。
      //----------------------------------------------------
      //--------------------------------------------------------
      // CDC通信パラメータ初期化 と CDCの初期化
      //--------------------------------------------------------
      // 引数：
      // struct sParam_Communication_CDC
      //                  *pstructParam_Communication_CDC :
      //          Communication_CDC用パラメータの構造体のポインタ


      // 戻り値：
      //    なし
      //--------------------------------------------------------
      Initialize_Communication_CDC(pstructParam_Communication_CDC);

      pstructParam_Communication_CDC->int16_vCDC_Initialize = 1;
                                    // CDC初期化フラグON
    }

    pstructParam_Communication_CDC->uint16_vCDC_SendLength = 0;
                                    // 送信データ長クリア


    STC_uint8_JobNum_CDC = 0;  // CDC通信処理番号初期化


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //------------------------------------------------------
    // CDCによるVCP受信処理
    //------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // struct sParam_Communication_CDC
    //                  *pstructParam_Communication_CDC :
    //          Communication_CDC用パラメータの構造体のポインタ


    // 戻り値：
    //   -1 : 受信なし
    //    0 : 受信なし
    //    1以上 : 受信byte数 : 受信終了
    //------------------------------------------------------
    int16_ReceiveLength = Receive_CDC(&STC_uint8_JobNum_CDC,
                            pstructParam_Communication_CDC);
    if(int16_ReceiveLength <= 0){
      // 受信なしの場合
      int16_Return = -1;  // 処理継続
      break;  // 処理番号維持 : 現在の処理番号をループ
    }
    //--------------------------------------------------
    // 受信を行うまで、この下に行きません。
    //--------------------------------------------------


    //----------------------------------------------------------
    // 受信コマンドを実行する。 : LED点滅
    //----------------------------------------------------------
    // 引数 ：
    // uint16_t uint16_ReceiveLength : 受信データ数
    // uint8_t *puint8_aReceiveData :
    //          受信データが格納されているBufferのポインタ
    // uint8_t *puint8_SendData :
    //          応答送信データを格納するBufferのポインタ

    // struct sParam_Blink_LED_ST *pstructParam_Blink_LED_ST :
    //              ST LED点滅用パラメータの構造体のポインタ


    // 戻り値 ： 応答送信データ数
    //----------------------------------------------------------
    pstructParam_Communication_CDC->uint16_vCDC_SendLength
        = Execute_Command_LED((uint16_t)int16_ReceiveLength,
                              GLB_uint8_aCDC_ReceiveData,
                              GLB_uint8_aCDC_SendData,
                              &GLB_structParam_Blink_LED_ST);
    if(pstructParam_Communication_CDC->
                        uint16_vCDC_SendLength == 0){
      //--------------------------------------------
      // 送信データなしの場合
      //--------------------------------------------
      *puint8_JobNum = 0;  // 処理番号初期化
      int16_Return = 0;    // OK終了
      break;
    }


    //----------------------------------------------
    // 送信データがある場合
    //----------------------------------------------
    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //----------------------------------------------------------
    // CDC送信セマフォチェック :
    // 他のモジュールでCDC送信を実行していないかをチェックします。
    //----------------------------------------------------------
    if(pstructParam_Communication_CDC->
                        int16_vSemaphore_CDC_Send != 0){
      // 使用中
      int16_Return = -1;  // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }
    //------------------------------------------------------
    // 他のモジュールが GLB_int16_vSemaphore_CDC_Send を
    // 開放するまで、この下に行きません。
    //------------------------------------------------------


    pstructParam_Communication_CDC->
        int16_vSemaphore_CDC_Send = 1;  // CDC送信セマフォ取得


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理中
    break;

  case 3:
    //----------------------------------------------------------
    // CDCによるVCP送信処理
    //----------------------------------------------------------
    // 引数：
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // uint16_t uint16_SendLength : 送信データ数
    // uint8_t *puint8_SendData :
    //              送信データが格納されているBufferのポインタ

    // 戻り値：
    //   -1 : 送信中
    //    0 : 送信終了 : OK
    //    1 : エラー
    //----------------------------------------------------------
    int16_Return = Send_CDC(&STC_uint8_JobNum_CDC,
        pstructParam_Communication_CDC->uint16_vCDC_SendLength,
                            GLB_uint8_aCDC_SendData);
    if(int16_Return == -1){
      // 送信中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }
    //----------------------------------------------
    // 送信が終了するまで、この下に行きません。
    //----------------------------------------------


    pstructParam_Communication_CDC->uint16_vCDC_SendLength = 0;
                                        // 送信データ長クリア

    pstructParam_Communication_CDC->int16_vSemaphore_CDC_Send = 0;
                                        // CDC送信セマフォ解放


    *puint8_JobNum = 0;  // 処理番号初期化
    // int16_Return : 処理結果
    // 処理終了
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}



//----------------------------------------------------------
// CDCによるVCP送信処理
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// uint16_t uint16_SendLength : 送信データ数
// uint8_t *puint8_SendData :
//              送信データが格納されているBufferのポインタ


// 戻り値：
//   -1 : 送信中
//    0 : 送信終了 : OK
//    1 : エラー
//----------------------------------------------------------
int16_t Send_CDC(uint8_t *puint8_JobNum,
                  uint16_t uint16_SendLength,
                  uint8_t *puint8_SendData)
{
  int16_t int16_Return;

  uint8_t uint8_Result;


  switch(*puint8_JobNum)
  {
  case 0:
    // CDC送信
    uint8_Result
        = CDC_Transmit_FS(puint8_SendData, uint16_SendLength);
    if(uint8_Result == USBD_BUSY){
      int16_Return = -1;  // 処理継続
      break;  // 処理番号維持 : case 0 を繰り返し実行します。
    }
    //--------------------------------------------------
    // 一回分の送信が終了するまで、この下に行きません。
    //--------------------------------------------------


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //----------------------------------------------
    // 送信終了後の処理がある場合、ここに記述する。
    //----------------------------------------------


    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 0;    // OK終了
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}



//----------------------------------------------------------
// CDCによるVCP受信処理
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//     -1 : 受信処理中
//      0 : 受信なし
//  1以上 : 受信byte数 : 受信終了
//----------------------------------------------------------
int16_t Receive_CDC(uint8_t *puint8_JobNum,
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC)
{
  int16_t int16_Return;


  switch(*puint8_JobNum){
  case 0:
    //----------------------------------------------
    // 初期化処理がある場合、ここに記述します。
    //----------------------------------------------
    pstructParam_Communication_CDC->
        int16_vTimer_CDC_Rcv_End = -1;  // 受信OFF状態をセット

    pstructParam_Communication_CDC->uint16_vLength_CDC_Rcv = 0;
                                    // CDC受信データ数クリア


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    //------------------------------------------------------
    // CDC受信が発生したときに、
    // Communication_CDC用パラメータ構造体の
    // int16_vTimer_CDC_Rcv_End に値をセットする。
    //                 (def_Timer_CDC_Rcv_End)

    // System tick timer割り込み内でデクリメントされ、
    // 10mSec間、次の受信がなかった場合、受信終了と判断する。
    //------------------------------------------------------
    if(pstructParam_Communication_CDC->
                        int16_vTimer_CDC_Rcv_End != 0){
      int16_Return = -1;  // 処理継続
      break;  // 処理番号維持 : 現在の処理番号をループ
    }
    //------------------------------------------------------
    // pstructParam_Communication_CDC->
    //          int16_vTimer_CDC_Rcv_Endが 0 になるまで、
    // この下には行きません。
    //------------------------------------------------------


    //------------------------------------------------------
    // CDC受信終了
    //------------------------------------------------------
    pstructParam_Communication_CDC->
        int16_vTimer_CDC_Rcv_End = -1;  // 受信OFF状態をセット

    int16_Return
        = (int16_t)pstructParam_Communication_CDC->
                                    uint16_vLength_CDC_Rcv;
                                    // 受信データ数セット
    pstructParam_Communication_CDC->uint16_vLength_CDC_Rcv = 0;
                                    // CDC受信データ数クリア


    *puint8_JobNum = 0;  // 処理番号初期化
    // int16_Return : 受信データ数 : 一回受信終了
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 0;    // NG
    break;
  }

  return(int16_Return);
}
