/*
 * Communication_CDC.h
 *
 *  Created on: 2025/01/09
 */

#ifndef COMMUNICTION_CDC_H_
#define COMMUNICTION_CDC_H_

#include "Dev_Conf.h"

#include def_Header_HAL

#include "usbd_cdc_if.h"



//--------------------------------------
// 受信終了タイマの値
//--------------------------------------
#define def_Timer_CDC_Rcv_End  20
//--------------------------------------



// Communication_CDC用パラメータの構造体
struct sParam_Communication_CDC{
  // CDC受信終了タイマ
  volatile int16_t int16_vTimer_CDC_Rcv_End;
  // CDC受信データ数
  volatile uint16_t uint16_vLength_CDC_Rcv;

  // CDC送信セマフォ
  volatile int16_t int16_vSemaphore_CDC_Send;

  // CDC送信データ数
  volatile uint16_t uint16_vCDC_SendLength;
  // CDC受信データ数
  volatile uint16_t uint16_vCDC_RcvLength;


  //------------------------------------------------
  // CDC初期化フラグ
  //------------------------------------------------
  //  0以下: 初期化されていない。
  //  1以上: 初期化済み
  //------------------------------------------------
  volatile int16_t int16_vCDC_Initialize;
};



//----------------------------------------------------------
// Communication_CDC用パラメータの構造体の初期化
//----------------------------------------------------------
// 引数：
// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//    なし
//----------------------------------------------------------
void Initialize_sParam_Communication_CDC(
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC);



//----------------------------------------------------------
// CDC通信パラメータ初期化 と CDCの初期化
//----------------------------------------------------------
// 引数：
// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//    なし
//----------------------------------------------------------
void Initialize_Communication_CDC(
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC);



//----------------------------------------------------------
// CDC通信処理
//----------------------------------------------------------
// 処理内容 :

// 1. CDCの初期化がされていない場合、初期化します。
// 2. 受信が発生するのを待ち、受信データを取得します。
// 3. 受信データを判定して、内容に従い処理を行います。
// 4. 受信データによる処理の結果、
//      送信データがセットされた場合、送信処理を行います。
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Communication_CDC(uint8_t *puint8_JobNum,
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC);



//----------------------------------------------------------
// CDCによるVCP送信処理
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// uint16_t uint16_SendLength : 送信データ数
// uint8_t *puint8_SendData :
//              送信データが格納されているBufferのポインタ


// 戻り値：
//   -1 : 送信中
//    0 : 送信終了 : OK
//    1 : エラー
//----------------------------------------------------------
int16_t Send_CDC(uint8_t *puint8_JobNum,
                 uint16_t uint16_SendLength,
                 uint8_t *puint8_SendData);



//----------------------------------------------------------
// CDCによるVCP受信処理
//----------------------------------------------------------
// 引数：
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// struct sParam_Communication_CDC
//                  *pstructParam_Communication_CDC :
//          Communication_CDC用パラメータの構造体のポインタ


// 戻り値：
//    -1 : 受信処理中
//     0 : 受信なし
// 1以上 : 受信byte数 : 受信終了
//----------------------------------------------------------
int16_t Receive_CDC(uint8_t *puint8_JobNum,
                     struct sParam_Communication_CDC
                            *pstructParam_Communication_CDC);



//------------------------------------------------------
// Communication_CDC用パラメータの構造体の宣言
//------------------------------------------------------
extern struct sParam_Communication_CDC
                    GLB_structParam_Communication_CDC;


// CDC通信処理用 送信データ
extern uint8_t GLB_uint8_aCDC_SendData[APP_TX_DATA_SIZE];

// CDC通信処理用 受信データ
extern uint8_t GLB_uint8_aCDC_ReceiveData[APP_RX_DATA_SIZE];


#endif /* COMMUNICTION_CDC_H_ */
